@@ -1,5 +1,6 @@
 from trex.stl.api import *
 from snappi_trex.valueoptions import ValueOptions
+from snappi_trex.validation import Validation
 
 class SetConfig:
 
@@ -10,6 +11,7 @@ class SetConfig:
         args: 
             - rate: A dictionary object containing all of the rate configuration info
         """
+        Validation.validate_rate(rate)
         pps = bps = percent = None
         if rate['choice'] == 'pps':
             pps = rate['pps']
@@ -23,8 +25,6 @@ class SetConfig:
             bps = rate['gbps'] * 1000000000
         elif rate['choice'] == 'percentage':
             percent = rate['percentage']
-        else:
-            raise STLError('Invalid rate option')
         return pps, bps, percent
 
     
@@ -38,28 +38,30 @@ class SetConfig:
             - percent: percent of layer 2 bit rate
             Note: Only one of (pps, bps, percent) will have a value. The rest are None.
         """
+        Validation.validate_duration(duration)
         if duration['choice'] == 'fixed_packets':
             mode = STLTXSingleBurst(
                 total_pkts=duration['fixed_packets']['packets'], 
                 pps=pps, bps_L2=bps, percentage=percent
             )
 
-        elif duration['choice'] == 'fixed_seconds':
-            raise STLError('T-Rex does not support fixed_seconds duration option')
-
         elif duration['choice'] == 'continuous':
             mode = STLTXCont(pps=pps, bps_L2=bps, percentage=percent)
 
         elif duration['choice'] == 'burst':
+            # TODO: Fix the ibg param to inter burst gap, not gap
+            ibg = 0.0
+            if 'inter_burst_gap' in duration['burst']:
+                if duration['burst']['inter_burst_gap']['choice'] == 'nanoseconds':
+                    ibg = duration['burst']['inter_burst_gap']['nanoseconds']
+                elif duration['burst']['inter_burst_gap']['choice'] == 'microseconds':
+                    ibg = duration['burst']['inter_burst_gap']['microseconds'] / 1000
             mode = STLTXMultiBurst(
                 pkts_per_burst=duration['burst']['packets'],
-                ibg=duration['burst']['gap'],
+                ibg=ibg,
                 count=duration['burst']['bursts'],
                 pps=pps, bps_L2=bps, percentage=percent)
 
-        else:
-            raise STLError('Invalid duration option')
-
         return mode
 
     
@@ -73,6 +75,7 @@ class SetConfig:
             - packet_headers: An array of objects that represent packet headers and all
             of their fields
         """
+        Validation.validate_packet(packet_headers)
         pkt_headers = []
         vm_cmds = []
         layers = [] # Keeps track of all of the layer types in order
@@ -106,12 +109,67 @@ class SetConfig:
                         layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['src'], 
                         length=32, field_str='src'
                     )
-
                 if 'dst' in header['ipv4']:
                     vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
                         layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['dst'], 
                         length=32, field_str='dst'
                     )
+                if 'version' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['version'], 
+                        length=4, field_str='version'
+                    )
+                if 'header_length' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['header_length'], 
+                        length=4, field_str='ihl', bit_fixup=4
+                    )
+                if 'priority' in header['ipv4']:
+                    if 'raw' in header['ipv4']['priority']:
+                        vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                            layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['priority']['raw'], 
+                            length=8, field_str='tos'
+                        )
+                if 'total_length' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['total_length'], 
+                        length=16, field_str='len'
+                    )
+                if 'identification' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['identification'], 
+                        length=16, field_str='id'
+                    )
+                if 'reserved' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['reserved'], 
+                        length=1, field_str='flags', bit_fixup=0
+                    )
+                if 'dont_fragment' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['dont_fragment'], 
+                        length=1, field_str='flags', bit_fixup=1
+                    )
+                if 'more_fragments' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['more_fragments'], 
+                        length=1, field_str='flags', bit_fixup=2
+                    )
+                if 'fragment_offset' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['fragment_offset'], 
+                        length=13, field_str='frag', bit_fixup=3
+                    )
+                if 'time_to_live' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['time_to_live'], 
+                        length=8, field_str='ttl'
+                    )
+                if 'protocol' in header['ipv4']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='IP', 
+                        layer_cnt=layer_cnt['IP'], header_field=header['ipv4']['protocol'], 
+                        length=8, field_str='proto'
+                    )
 
             # UDP HEADER FIELDS CONFIGURATION
             elif header['choice'] == 'udp':
@@ -164,14 +222,56 @@ class SetConfig:
                         layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['data_offset'], 
                         length=4, field_str='dataofs'
                     )
+                if 'ecn_ns' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ecn_ns'], 
+                        length=1, field_str='flags', bit_fixup=7
+                    )
+                if 'ecn_cwr' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ecn_cwr'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=0
+                    )
+                if 'ecn_echo' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ecn_echo'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=1
+                    )
+                if 'ctl_urg' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ctl_urg'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=2
+                    )
+                if 'ctl_ack' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ctl_ack'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=3
+                    )
+                if 'ctl_psh' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ctl_psh'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=4
+                    )
+                if 'ctl_rst' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ctl_rst'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=5
+                    )
+                if 'ctl_syn' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ctl_syn'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=6
+                    )
+                if 'ctl_fin' in header['tcp']:
+                    vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
+                        layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['ctl_fin'], 
+                        length=1, field_str='flags', fixup=1, bit_fixup=7
+                    )
                 if 'window' in header['tcp']:
                     vm_cmds += ValueOptions.get_value_cmds(layer_type='TCP', 
                         layer_cnt=layer_cnt['TCP'], header_field=header['tcp']['window'], 
                         length=16, field_str='window'
                     )
-
-            else:
-                raise STLError('Invalid packet header option')
             
         return vm_cmds, pkt_headers, layers
 
@@ -186,6 +286,7 @@ class SetConfig:
             - layers: An ordered list of strings representing the order of headers on the packet base
                     Strings must conform to supported Scapy protocols
         """
+        Validation.validate_size(f_size)
         vm_cmds = []
         if f_size['choice'] == 'increment':
             needs_trim = True
@@ -226,4 +327,4 @@ class SetConfig:
         # Fill the rest of the packet with x's
         pad = max(0, max_pkt_size - len(pkt_base)) * 'x'
 
-        return vm_cmds, pad
\ No newline at end of file
+        return vm_cmds, pad
