@@ -30,7 +30,7 @@ class Api(snappi.Api):
             host='https://127.0.0.1:11009' if host is None else host
         )
         self._c = STLClient()
-        self._portIndices = {}
+        self._portIndices = []
         try:
             # connect to server
             self._c.connect()
@@ -51,20 +51,22 @@ class Api(snappi.Api):
         if 'ports' in self._cfg:
             i = 0
             for p in self._cfg['ports']:
-                self._portIndices[p['name']] = i
-                i += 1
+                self._portIndices.append(p['name'])
 
 
     def set_config(self, config):
         """Set or update the configuration
         """
+        # Create response
+        res = {'warnings': []}
+
         # print(config.serialize())
         self._cfg = json.loads(config.serialize())
         self._loadPorts()
 
         try:
             # prepare our ports
-            self._c.reset(ports = list(self._portIndices.values()))
+            self._c.reset(ports = list(range(len(self._portIndices))))
 
             # for each Snappi flow, construct the equivalent T-Rex stream
             for f in self._cfg["flows"]:
@@ -95,6 +97,7 @@ class Api(snappi.Api):
                 
                 # TODO: Now fix the checksum of modified packets
                 
+
                 # Construct the packet with given Flow Variables
                 vm = STLScVmRaw(vmCmds)
                 pkt = STLPktBuilder(pkt = pkt_base/pad, vm = vm)
@@ -104,14 +107,15 @@ class Api(snappi.Api):
                             mode = mode)
 
                 # Add the stream to the client
-                self._c.add_streams([s1], ports=[self._portIndices[f['tx_rx']['port']['tx_name']]])
+                self._c.add_streams([s1], ports=[self._portIndices.index(f['tx_rx']['port']['tx_name'])])
 
         # Disconnect on error
         except STLError as e:
+            res = {'errors': [e]}
             self._c.disconnect()
             print(e)
 
-        return {'warnings': []}
+        return res
 
 
     def set_transmit_state(self, payload):
@@ -122,18 +126,24 @@ class Api(snappi.Api):
             self._c.clear_stats()
 
             # Start streams on all ports
-            self._c.start(ports = list(self._portIndices.values()))
+            self._c.start(ports = list(range(len(self._portIndices))))
 
             # Wait until traffic stops
-            self._c.wait_on_traffic(ports = list(self._portIndices.values()))
+            self._c.wait_on_traffic(ports = list(range(len(self._portIndices))))
 
         except STLError as e:
+            self._c.disconnect()
             print(e)
 
     
     def set_capture_state(self, payload):
         """Starts capture on all ports that have capture enabled.
         """
+        try:
+            """"""
+        except STLError as e:
+            self._c.disconnect()
+            print(e)
         
 
     def get_capture(self, request):
