@@ -1,4 +1,5 @@
 from trex.stl.api import *
+from snappi_trex.util import Util
 
 
 class ValueOptions:
@@ -7,24 +8,26 @@ class ValueOptions:
     This class also contains useful utility functions for configuration
     """
 
-    def get_value_cmds(layer_type, layer_cnt, header_field, length, field_str, fixup=0):
+    def get_value_cmds(layer_type, layer_cnt, header_field, length, field_str, fixup=0, bit_fixup=0):
         """
         Returns the set of VM instructions to configure a specific header field
         to a value option. 
         args:
             - layer_type: A string representing the header type (Must conform to Scapy protocol types)
             - layer_cnt: An int representing the number of occurrences of 
-                        this layer type in the packet base
+                this layer type in the packet base
             - header_field: A dictionary object that contains the value configuration 
-                        information for the given header field
+                information for the given header field
             - length: An int representing the length of the header field in bits
             - field_str: A string representing the field (Must conform to Scapy protocol fields)
             - fixup: An int representing how many packet bytes to go forward when writing the field bytes.
-                        (Positive fixup means forward, negative means backward)
+                (Positive fixup means forward, negative means backward)
+            - bit_fixup: An int representing how many packet bits to go forward when writing the field bits.
+                (Use this when the header field starts in the middle of a byte)
         """
-        varname = "{0}{1}_{2}".format(layer_type, layer_cnt-1, field_str)
+        varname = "{0}{1}_{2}:{3}".format(layer_type, layer_cnt-1, field_str, 8*fixup + bit_fixup)
         vm_cmds = []
-        mask = ValueOptions.get_mask(length)
+        mask = Util.get_mask(length)
 
         # If length is more than 4 bytes, call get_big_value_cmds
         if length > 32:
@@ -42,7 +45,7 @@ class ValueOptions:
         # Handle every source port value option: Value, values, inc, dec
         if header_field['choice'] == 'value':
             add_val = 0
-            val = ValueOptions.convert_to_long(header_field['value'], layer_type)
+            val = Util.convert_to_long(header_field['value'], layer_type)
             vm_cmds.append(STLVmFlowVar(name=varname, size=numBytes,
                 min_value=val, max_value=val, step=1, op='inc'
             ))
@@ -51,14 +54,14 @@ class ValueOptions:
             add_val = 0
             value_list = []
             for v in header_field['values']:
-                value_list.append(ValueOptions.convert_to_long(v, layer_type))
+                value_list.append(Util.convert_to_long(v, layer_type))
             vm_cmds.append(STLVmFlowVar(name=varname, size=numBytes,
                 value_list=value_list
             ))
 
         elif header_field['choice'] == 'increment':
-            start = ValueOptions.convert_to_long(header_field['increment']['start'], layer_type)
-            step = ValueOptions.convert_to_long(header_field['increment']['step'], layer_type)
+            start = Util.convert_to_long(header_field['increment']['start'], layer_type)
+            step = Util.convert_to_long(header_field['increment']['step'], layer_type)
             cnt = header_field['increment']['count']
             minV, maxV, init, add_val = ValueOptions.inc_values(start,step,cnt,length)
             vm_cmds.append(STLVmFlowVar(name=varname, size=numBytes, init_value = init,
@@ -66,8 +69,8 @@ class ValueOptions:
             ))
 
         elif header_field['choice'] == 'decrement':
-            start = ValueOptions.convert_to_long(header_field['decrement']['start'], layer_type)
-            step = ValueOptions.convert_to_long(header_field['decrement']['step'], layer_type)
+            start = Util.convert_to_long(header_field['decrement']['start'], layer_type)
+            step = Util.convert_to_long(header_field['decrement']['step'], layer_type)
             cnt = header_field['decrement']['count']
             minV, maxV, init, add_val = ValueOptions.inc_values(start,-step,cnt,length)
             vm_cmds.append(STLVmFlowVar(name=varname, size=numBytes, init_value = init,
@@ -79,8 +82,8 @@ class ValueOptions:
         
         # Write the first n bits of the variable to the first n bits of the packet field
         pkt_offset = "{0}:{1}.{2}".format(layer_type, layer_cnt-1, field_str)
-        bit_shift = numBytes*8-length
-        add_val_int = ValueOptions.uint_to_int(add_val, 32)
+        bit_shift = numBytes*8 - length - bit_fixup
+        add_val_int = Util.uint_to_int(add_val, 32)
         vm_cmds.append(STLVmWrMaskFlowVar(fv_name=varname, 
                                     pkt_cast_size=numBytes,
                                     mask=mask << bit_shift,
@@ -98,12 +101,12 @@ class ValueOptions:
         """
         varname = "{0}{1}_{2}".format(layer_type, layer_cnt-1, field_str)
         vm_cmds = []
-        mask = ValueOptions.get_mask(length) 
+        mask = Util.get_mask(length) 
 
         # Handle every source port value option: Value, values, inc, dec
         if header_field['choice'] == 'value':
             add_val = 0
-            val = ValueOptions.convert_to_long(header_field['value'], layer_type)
+            val = Util.convert_to_long(header_field['value'], layer_type)
             vm_cmds.append(STLVmFlowVar(name=varname, size=8,
                 min_value=val, max_value=val, step=1, op='inc'
             ))
@@ -112,27 +115,27 @@ class ValueOptions:
             add_val = 0
             value_list = []
             for v in header_field['values']:
-                value_list.append(ValueOptions.convert_to_long(v, layer_type))
+                value_list.append(Util.convert_to_long(v, layer_type))
 
             vm_cmds.append(STLVmFlowVar(name=varname, size=8,
                 value_list=value_list
             ))
 
         elif header_field['choice'] == 'increment':
-            start = ValueOptions.convert_to_long(header_field['increment']['start'], layer_type)
-            step = ValueOptions.convert_to_long(header_field['increment']['step'], layer_type)
+            start = Util.convert_to_long(header_field['increment']['start'], layer_type)
+            step = Util.convert_to_long(header_field['increment']['step'], layer_type)
             cnt = header_field['increment']['count']
-            minV, maxV, init, add_val = ValueOptions.inc_values(start,step,cnt,length)
+            minV, maxV, init, add_val = ValueOptions.inc_values_big(start,step,cnt,length)
             
             vm_cmds.append(STLVmFlowVar(name=varname, size=8, init_value = init,
                 min_value=minV, max_value=maxV, step=step, op='inc'
             ))
 
         elif header_field['choice'] == 'decrement':
-            start = ValueOptions.convert_to_long(header_field['decrement']['start'], layer_type)
-            step = ValueOptions.convert_to_long(header_field['decrement']['step'], layer_type)
+            start = Util.convert_to_long(header_field['decrement']['start'], layer_type)
+            step = Util.convert_to_long(header_field['decrement']['step'], layer_type)
             cnt = header_field['decrement']['count']
-            minV, maxV, init, add_val = ValueOptions.inc_values(start,-step,cnt,length)
+            minV, maxV, init, add_val = ValueOptions.inc_values_big(start,-step,cnt,length)
             
             vm_cmds.append(STLVmFlowVar(name=varname, size=8, init_value = init,
                 min_value=minV, max_value=maxV, step=step, op='dec'
@@ -146,7 +149,7 @@ class ValueOptions:
         bit_mask = mask >> 32 << bit_shift
 
         # Write first n-32 bits of data
-        add_val_top = ValueOptions.uint_to_int(add_val>>32, 32)
+        add_val_top = Util.uint_to_int(add_val>>32, 32)
         vm_cmds.append(STLVmWrMaskFlowVar(fv_name=varname, 
                                 pkt_cast_size=4,
                                 mask=bit_mask,
@@ -157,7 +160,7 @@ class ValueOptions:
 
         # Write last 4 bytes
         num_bytes = (length+7)//8
-        add_val_bot = ValueOptions.uint_to_int(add_val, 32)
+        add_val_bot = Util.uint_to_int(add_val, 32)
         for i in range(3, -1, -1):
             vm_cmds.append(STLVmWrMaskFlowVar(fv_name=varname, 
                                     is_big=False,
@@ -169,6 +172,7 @@ class ValueOptions:
         return vm_cmds
 
 
+
     def inc_values(start, step, count, length):
         """
         Returns the min, max, initial, and add value for a given start, step, 
@@ -190,7 +194,7 @@ class ValueOptions:
 
         if step < 0: # Decrementing
             step = -((-step) % (1<<length))
-            max = init = ValueOptions.get_mask(length)
+            max = init = Util.get_mask(length)
             min = max + count * step
             add_val = (start + 1) % (1<<length)
 
@@ -203,52 +207,38 @@ class ValueOptions:
         return (min, max, init, add_val)
 
 
-    def get_mask(bits):
+    def inc_values_big(start, step, count, length):
         """
-        returns a mask of 1's of size bits
-        """
-        result = (1 << bits) - 1
-        return result
-
-    
-    def uint_to_int(uint, size):
-        """
-        converts an unsigned int of size n to a signed int of size n
-        """
-        result = uint & ValueOptions.get_mask(size)
-        if result >= (1 << (size-1)):
-            result -= (1 << size)
-        return result
-
-
-    def convert_to_long(value, layer_type):
-        """
-        Parses address values into a long int
+        Returns the min, max, initial, and add value for a given start, step, count, 
+        and mask value for incrementing or decrementing fields larger than 4 bytes. 
+        The min value is either the bottom value of the decrement or the initial value 
+        of the increment. The max value is either the top value of the increment or
+        the initial value of the decrement. The add value is how much needs to be 
+        added in order to correct the start value
         args:
-            - value: A string or long representing the address
-            - layer_type: A string representing the layer type
+            - start: int representing start value of the increment or decrement
+            - step: int representing how much to increment or decrement by.
+                    (Positive means increment, negative means decrement)
+            - count: number of increments before cycling to the start
+            - length: the field length
         """
-        if isinstance(value, int): 
-            return value
 
-        if layer_type == 'IP':
-            elements = value.split('.')
-            result = 0
-            for e in elements:
-                result = result << 8
-                if len(e) == 0:
-                    continue
-                result += int(e)
-            return result
+        if abs(step) * count >= (1 << length):
+            raise STLError('incrementing/decrementing count is too high.' + 
+            'step*count must be at most {0} bits long'.format(length))
 
-        if layer_type == 'Ethernet':
-            elements = value.split(':')
-            result = 0
-            for e in elements:
-                result = result << 8
-                if len(e) == 0:
-                    continue
-                result += int(e, 16)
-            return result
+        if step < 0: # Decrementing
+            maxV = init = start | (Util.get_mask(64-length) << length)
+            minV = maxV + step * count
+            if minV < 0:
+                raise STLError('incrementing/decrementing count is too high.' + 
+                'overflow for 8 byte fields is not supported')
+
+        else: # Incrementing
+            minV = init = start
+            maxV = minV + step * count
+            if maxV > Util.get_mask(64):
+                raise STLError('incrementing/decrementing count is too high.' + 
+                'overflow for 8 byte fields is not supported')
         
-        return value
\ No newline at end of file
+        return (minV, maxV, init, 0)
